#!/usr/bin/env python

"""
Web user interface operations.

Copyright (C) 2014, 2015, 2017 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.editing import FormDate, FormPeriod

# Form field extraction and serialisation.

def get_date_control_inputs(args, name, tzid_name=None):

    """
    Return a tuple of date control inputs taken from 'args' for field names
    starting with 'name'.

    If 'tzid_name' is specified, the time zone information will be acquired
    from fields starting with 'tzid_name' instead of 'name'.
    """

    return args.get("%s-date" % name, []), \
           args.get("%s-hour" % name, []), \
           args.get("%s-minute" % name, []), \
           args.get("%s-second" % name, []), \
           args.get("%s-tzid" % (tzid_name or name), [])

def get_date_control_values(args, name, multiple=False, tzid_name=None, tzid=None):

    """
    Return a form date object representing fields taken from 'args' starting
    with 'name'.

    If 'multiple' is set to a true value, many date objects will be returned
    corresponding to a collection of datetimes.

    If 'tzid_name' is specified, the time zone information will be acquired
    from fields starting with 'tzid_name' instead of 'name'.

    If 'tzid' is specified, it will provide the time zone where no explicit
    time zone information is indicated in the field data.
    """

    dates, hours, minutes, seconds, tzids = get_date_control_inputs(args, name, tzid_name)

    # Handle absent values by employing None values.

    field_values = map(None, dates, hours, minutes, seconds, tzids)

    if not field_values and not multiple:
        all_values = FormDate()
    else:
        all_values = []
        for date, hour, minute, second, tzid_field in field_values:
            value = FormDate(date, hour, minute, second, tzid_field or tzid)

            # Return a single value or append to a collection of all values.

            if not multiple:
                return value
            else:
                all_values.append(value)

    return all_values

def set_date_control_values(formdates, args, name, tzid_name=None):

    """
    Using the values of the given 'formdates', replace form fields in 'args'
    starting with 'name'.

    If 'tzid_name' is specified, the time zone information will be stored in
    fields starting with 'tzid_name' instead of 'name'.
    """

    args["%s-date" % name] = []
    args["%s-hour" % name] = []
    args["%s-minute" % name] = []
    args["%s-second" % name] = []
    args["%s-tzid" % (tzid_name or name)] = []

    for d in formdates:
        args["%s-date" % name].append(d and d.date or "")
        args["%s-hour" % name].append(d and d.hour or "")
        args["%s-minute" % name].append(d and d.minute or "")
        args["%s-second" % name].append(d and d.second or "")
        args["%s-tzid" % (tzid_name or name)].append(d and d.tzid or "")

def get_period_control_values(args, start_name, end_name,
                              end_enabled_name, times_enabled_name,
                              origin=None, origin_name=None,
                              replacement_name=None, cancelled_name=None,
                              recurrenceid_name=None, tzid=None,
                              start_index=0):

    """
    Return period values from fields found in 'args' prefixed with the given
    'start_name' (for start dates), 'end_name' (for end dates),
    'end_enabled_name' (to enable end dates for periods), 'times_enabled_name'
    (to enable times for periods).

    If 'origin' is specified, a single period with the given origin is
    returned. If 'origin_name' is specified, fields containing the name will
    provide origin information.

    If specified, fields containing 'replacement_name' will indicate periods
    provided by separate recurrences, fields containing 'cancelled_name'
    will indicate periods that are replacements and cancelled, and fields
    containing 'recurrenceid_name' will indicate periods that have existing
    recurrence details from an event.

    If 'tzid' is specified, it will provide the time zone where no explicit
    time zone information is indicated in the field data.
    """

    # Get the end datetime and time presence settings.

    all_end_enabled = args.get(end_enabled_name, [])
    all_times_enabled = args.get(times_enabled_name, [])

    # Get the origins of period data and whether the periods are replacements.

    if origin:
        all_origins = [origin]
    else:
        all_origins = origin_name and args.get(origin_name, []) or []

    all_replacements = replacement_name and args.get(replacement_name, []) or []
    all_cancelled = cancelled_name and args.get(cancelled_name, []) or []
    all_recurrenceids = recurrenceid_name and args.get(recurrenceid_name, []) or []

    # Get the start and end datetimes.

    all_starts = get_date_control_values(args, start_name, True, tzid=tzid)
    all_ends = get_date_control_values(args, end_name, True, start_name, tzid=tzid)

    # Construct period objects for each start, end, origin combination.

    periods = []

    for index, (start, end, found_origin, recurrenceid) in \
        enumerate(map(None, all_starts, all_ends, all_origins, all_recurrenceids),
                  start_index):

        # Obtain period settings from separate controls.

        end_enabled = str(index) in all_end_enabled
        times_enabled = str(index) in all_times_enabled
        replacement = str(index) in all_replacements
        cancelled = str(index) in all_cancelled

        period = FormPeriod(start, end, end_enabled, times_enabled, tzid,
                            found_origin or origin, replacement, cancelled,
                            recurrenceid)
        periods.append(period)

    # Return a single period if a single origin was specified.

    if origin:
        return periods[0]
    else:
        return periods

def set_period_control_values(periods, args, start_name, end_name,
                              end_enabled_name, times_enabled_name,
                              origin_name=None, replacement_name=None,
                              cancelled_name=None, recurrenceid_name=None):

    """
    Using the given 'periods', replace form fields in 'args' prefixed with the
    given 'start_name' (for start dates), 'end_name' (for end dates),
    'end_enabled_name' (to enable end dates for periods), 'times_enabled_name'
    (to enable times for periods).

    If 'origin_name' is specified, fields containing the name will provide
    origin information, fields containing 'replacement_name' will indicate
    periods provided by separate recurrences, fields containing 'cancelled_name'
    will indicate periods that are replacements and cancelled, and fields
    containing 'recurrenceid_name' will indicate periods that have existing
    recurrence details from an event.
    """

    # Record period settings separately.

    args[end_enabled_name] = []
    args[times_enabled_name] = []

    # Record origin and replacement information if naming is defined.

    if origin_name:
        args[origin_name] = []

    if replacement_name:
        args[replacement_name] = []

    if cancelled_name:
        args[cancelled_name] = []

    if recurrenceid_name:
        args[recurrenceid_name] = []

    all_starts = []
    all_ends = []

    for index, period in enumerate(periods):

        # Encode period settings in controls.

        if period.end_enabled:
            args[end_enabled_name].append(str(index))
        if period.times_enabled:
            args[times_enabled_name].append(str(index))

        # Add origin information where controls are present to record it.

        if origin_name:
            args[origin_name].append(period.origin or "")

        # Add replacement information where controls are present to record it.

        if replacement_name and period.replacement:
            args[replacement_name].append(str(index))

        # Add cancelled recurrence information where controls are present to
        # record it.

        if cancelled_name and period.cancelled:
            args[cancelled_name].append(str(index))

        # Add recurrence identifiers where controls are present to record it.

        if recurrenceid_name:
            args[recurrenceid_name].append(period.recurrenceid or "")

        # Collect form date information for addition below.

        all_starts.append(period.get_form_start())
        all_ends.append(period.get_form_end())

    # Set the controls for the dates.

    set_date_control_values(all_starts, args, start_name)
    set_date_control_values(all_ends, args, end_name, tzid_name=start_name)



# Utilities.

def filter_duplicates(l):

    """
    Return collection 'l' filtered for duplicate values, retaining the given
    element ordering.
    """

    s = set()
    f = []

    for value in l:
        if value not in s:
            s.add(value)
            f.append(value)

    return f

def remove_from_collection(l, indexes, fn):

    """
    Remove from collection 'l' all values present at the given 'indexes' where
    'fn' applied to each referenced value returns a true value. Values where
    'fn' returns a false value are added to a list of deferred removals which is
    returned.
    """

    still_to_remove = []
    correction = 0

    for i in indexes:
        try:
            i = int(i) - correction
            value = l[i]
        except (IndexError, ValueError):
            continue

        if fn(value):
            del l[i]
            correction += 1
        else:
            still_to_remove.append(value)

    return still_to_remove

# vim: tabstop=4 expandtab shiftwidth=4
